; #########################################################################
;
;  Prepared by Raymond Filiatreault, August 2000
;
;  SINWAVE is a program to display a sine wave on the screen and
;  modify its amplitude and frequency according to user input.
;
;  It was written primarily to demonstrate the use of Mixed Integers to
;  avoid programming the coprocessor and still have sufficient accuracy
;  with integer mathematics to produce symmetrical graphics.
;
;  Updated May, 2002, to correct a "page fault" error when maximizing the
;  window.
;
; #########################################################################

      .386                   ; minimum processor needed for 32 bit
      .model flat, stdcall   ; FLAT memory model & STDCALL calling
      option casemap :none   ; set code to case sensitive

; #########################################################################

      include \masm32\include\windows.inc

      include \masm32\include\user32.inc
      include \masm32\include\kernel32.inc
      include \masm32\include\gdi32.inc
      include \masm32\include\comctl32.inc
      include \masm32\include\mix.inc
      
      includelib \masm32\lib\user32.lib
      includelib \masm32\lib\kernel32.lib
      includelib \masm32\lib\gdi32.lib
      includelib \masm32\lib\comctl32.lib
      includelib \masm32\lib\mix.lib

; #########################################################################

      szText MACRO Name, Text:VARARG
        LOCAL lbl
          jmp lbl
            Name db Text,0
          lbl:
        ENDM

      m2m MACRO M1, M2
        push M2
        pop  M1
      ENDM

      return MACRO arg
        mov eax, arg
        ret
      ENDM

; #########################################################################

        WinMain PROTO :DWORD,:DWORD,:DWORD,:DWORD
        WndProc PROTO :DWORD,:DWORD,:DWORD,:DWORD

; #########################################################################

    .data
        CommandLine     dd  0
        hWnd            dd  0
        hInstance       dd  0
        hBMP            dd  0
        hDC             dd  0
        hmDC            dd  0
        
        MainRec         dd  0,0     ;RECT structure to receive the
        MainRecR        dd  0       ;   dimensions of the client area
        MainRecB        dd  0

        allocated       dd  0       ;size of allocated memory
        lpBMP           dd  0       ;address of allocated memory for
                                    ;preparing a bitmap for screen output
        
        scrwd           dd  0       ;for initial screen size and toolbar
        scrht           dd  0
        
        clienttop       dd  0       ;to hold dimensions
        clientwd        dd  0       ;   of current
        clientht        dd  0       ;       client area
        
        scanline        dd  0       ;bytes per scan line

        oldline         dd  0       ;line of previous pixel
        oldedi          dd  0       ;offset of previous pixel
        oldal           dd  0       ;location of previous pixel
        newline         dd  0       ;line of current pixel
        
        xaxis           dd  0       ;counter when drawing sine wave
        centerline      dd  0       ;mid-way of client area height
        
        frequency       dd  4       ;a range of 1-20 will be allowed for the
                                    ;number of full cycles in steps of 1
        frequencyx      dd  0       ;for calculations
        amplitude       dd  30      ;a range of +100 to -100 will be allowed
                                    ;in steps of 2

        updown          db  0       ;flag for drawing curve
        
        szDisplayName   db  "**  SINE  WAVES  **",0
        szresize        db  "Grab the edges to resize the window",0
        AboutMsg        db  "To demonstrate the use of Mixed Integers"
                        db  10,13,"by Raymond Filiatreault, June 2002",0

; ########################### Inserted modules ############################

    include toolbar.asm         ;that one then includes the TBMACROS.ASM

; #########################################################################

    .code

start:
    invoke GetModuleHandle, NULL ; provides the instance handle
    mov hInstance, eax

    invoke GetCommandLine        ; provides the command line address
    mov CommandLine, eax

    invoke WinMain,hInstance,NULL,CommandLine,SW_SHOWDEFAULT
    
    invoke ExitProcess,eax       ; cleanup & return to operating system

; #########################################################################

WinMain proc hInst     :DWORD,
             hPrevInst :DWORD,
             CmdLine   :DWORD,
             CmdShow   :DWORD

        ;====================
        ; Put LOCALs on stack
        ;====================

        LOCAL wc   :WNDCLASSEX
        LOCAL msg  :MSG

        szText szClassName,"Sine_Class"
         
        ;==================================================
        ; Fill WNDCLASSEX structure with required variables
        ;==================================================

        mov wc.cbSize,         sizeof WNDCLASSEX
        mov wc.style,          CS_HREDRAW or CS_VREDRAW \
                               or CS_BYTEALIGNWINDOW or CS_OWNDC
        mov wc.lpfnWndProc,    offset WndProc      ; address of WndProc
        mov wc.cbClsExtra,     NULL
        mov wc.cbWndExtra,     NULL
        m2m wc.hInstance,      hInst               ; instance handle
        mov wc.hbrBackground,  NULL     ;COLOR_BTNFACE+1     ; system color
        mov wc.lpszMenuName,   NULL
        mov wc.lpszClassName,  offset szClassName  ; window class name
            invoke LoadIcon,hInst,200    ; icon ID   ; resource icon
        mov wc.hIcon,          eax
            invoke LoadCursor,NULL,IDC_ARROW         ; system cursor
        mov wc.hCursor,        eax
        mov wc.hIconSm,        0

        invoke RegisterClassEx, ADDR wc     ; register the window class
        invoke InitCommonControls

        ; ==================================
        ; Create the main application window
        ; ==================================

        invoke GetSystemMetrics,SM_CYSCREEN ; get screen height in pixels
        mov scrht,eax
        invoke GetSystemMetrics,SM_CXSCREEN ; get screen width in pixels
        mov scrwd,eax

        shr eax,2
        mov ebx,scrht
        shr ebx,2
        mov ecx,scrwd
        shr ecx,1
        mov edx,scrht
        shr edx,1

        invoke CreateWindowEx,WS_EX_OVERLAPPEDWINDOW,
                              ADDR szClassName,
                              ADDR szDisplayName,
                              WS_OVERLAPPEDWINDOW,
                              eax,ebx,ecx,edx,
                              NULL,NULL,
                              hInst,NULL

        mov   hWnd,eax

        invoke LoadMenu,hInst,600               ; menu ID
        invoke SetMenu,hWnd,eax

        invoke ShowWindow,hWnd,SW_SHOWNORMAL    ; display the window
        invoke UpdateWindow,hWnd                ; update the display

        invoke GetDC,hWnd
        mov hDC,eax
        invoke CreateCompatibleDC,NULL
        mov hmDC,eax

        ;################################################################
        ;
        ; Calculate memory required for a monochrome bitmap (1 bit/pixel)
        ; to cover the entire window.
        ;
        ;#################################################################
        
        mov eax,scrwd
        shr eax,3                   ;divide by 8 pixels/byte (1 bit = 1 pixel)
        add eax,2                   ;to insure enough memory
        mul scrht                   ;total screen area
        
        invoke LocalAlloc,NULL,eax  ;allocate required memory
        mov lpBMP,eax               ;save pointer to allocated memory

        call MainSection            ;prepare/display initialized graphic

        invoke TextOut,hDC,30,50,ADDR szresize,35

      ;===================================
      ; Loop until PostQuitMessage is sent
      ;===================================

    StartLoop:
      invoke GetMessage,ADDR msg,NULL,0,0         ; get each message
      cmp eax, 0                                  ; exit if GetMessage()
      je ExitLoop                                 ; returns zero
      invoke TranslateMessage, ADDR msg           ; translate it
      invoke DispatchMessage,  ADDR msg           ; send it to message proc
      jmp StartLoop
    ExitLoop:

      return msg.wParam

WinMain endp

; #########################################################################

WndProc proc hWin   :DWORD,
             uMsg   :DWORD,
             wParam :DWORD,
             lParam :DWORD

    LOCAL Ps     :PAINTSTRUCT

    .if uMsg == WM_COMMAND
    
    ;======== menu commands ========

        .if wParam == 1000          ;Exit
            invoke SendMessage,hWin,WM_SYSCOMMAND,SC_CLOSE,NULL

        .elseif wParam == 1900
            invoke MessageBox,hWin,ADDR AboutMsg,ADDR szDisplayName,
                            MB_ICONINFORMATION

        .elseif wParam == 50        ;UP arrow on tool bar
            mov eax,amplitude
            cmp eax,100
            jge @F                  ;disregard input if at max. amplitude
            add eax,2
            mov amplitude,eax       ;store new amplitude
            call MainSection        ;prepare/display new graphic
          @@:

        .elseif wParam == 51        ;DOWN arrow on tool bar
            mov eax,amplitude
            cmp eax,-100
            jle @F                  ;disregard input if at min. amplitude
            sub eax,2
            mov amplitude,eax       ;store new amplitude
            call MainSection        ;prepare/display new graphic
          @@:

        .elseif wParam == 53        ;RIGHT arrow on tool bar
            mov eax,frequency
            .if eax > 1             ;if the frequency has not yet reached
                                    ;the lower limit of 1 cycle
                dec frequency
                call MainSection    ;prepare/display new graphic
            .endif

        .elseif wParam == 52        ;LEFT arrow on tool bar
            mov eax,frequency
            .if eax < 20            ;if the frequency has not yet reached
                                    ;the upper limit of 20 cycles
                inc frequency
                call MainSection    ;prepare/display new graphic
            .endif

        .endif

    ;====== end menu commands ======

    .elseif uMsg == WM_CREATE

        invoke Do_ToolBar,hWin

    .elseif uMsg == WM_SIZE

        invoke SendMessage,hToolBar,TB_AUTOSIZE,0,0
        call MainSection            ;prepare/display new graphic if
                                    ;window size is altered

    .elseif uMsg == WM_PAINT
        .if lpBMP != 0
            invoke BeginPaint,hWin,ADDR Ps
            mov hDC, eax
            
            call newscreen      ;repaint the client area from existing data
            invoke EndPaint,hWin,ADDR Ps
            return 0
        .endif

    .elseif uMsg == WM_CLOSE
    
        invoke LocalFree,lpBMP      ;CLEAN UP before closing
        invoke DeleteDC,hmDC

    .elseif uMsg == WM_DESTROY

        invoke PostQuitMessage,NULL
        return 0 
    .endif

    invoke DefWindowProc,hWin,uMsg,wParam,lParam

    ret

WndProc endp

; ########################################################################
;
; This section computes the location for each pixel of the sine wave
; according to the dimensions of the client area and the amplitude/frequency
; parameters modified by the user.
;
;##########################################################################

MainSection:

        .if lpBMP == 0
            ret             ;disregard if memory has not yet been allocated
        .endif

        call clientXY       ;get dimensions of client area
        cmp clientht,10
        jge @F
        ret                 ;disregard if available height is < 10

        ;--------------------------------------------------------------
        ; scan lines in the bitmap must end on a WORD (16-bit) boundary
        ;--------------------------------------------------------------
    @@:
        push edi

        mov eax,clientwd
        mov cl,16
        div cl              ;get the number of complete 16 bits

        .if ah != 0         ;if the remainder is not zero
            inc al          ;   another WORD will be needed
        .endif
        movzx eax,al        ;keep only the value in AL

        shl eax,1           ;gets number of BYTES per scan line
        mov scanline,eax

        ;--------------------------------------------------------------
        ;Fill the memory required by the client area with white bits
        ;--------------------------------------------------------------

        inc eax
        mul clientht        ;gives total BYTES required for client area
        mov ecx,eax
        shr ecx,2           ;divide by 4 bytes/dword
        mov eax,-1          ;for white color
        mov edi,lpBMP
    rep stosd               ;fill bitmap buffer with white background

        ;---------------------------------------------------------------
        ;In order to maintain some semblance of a sine curve when the
        ;width of the window is reduced with a high number of cycles,
        ;a minimum of 16 pixels per cycle will be retained for drawing
        ;the curves.
        ;---------------------------------------------------------------

        mov eax,frequency
        shl eax,4                   ;*16
        .if eax > clientwd          ;if client width not sufficient
            invoke MixDiv,clientwd,16   ;get number of available cycle in MI format
            mov frequencyx,eax      ;store result
        .else
            mov eax,frequency
            shl eax,16              ;convert it to Mixed Integer (MI) format
            mov frequencyx,eax      ;store result
        .endif

        ;---------------------------------------------------------------
        ;Each pixel along the x axis will be equivalent to a given angle
        ;according to the lenth of the x axis and the number of cycles.
        ;After computing the equivalent angle, its sine will be used to
        ;compute value on the y axis according to the amplitude required
        ;and length of the y axis. Those values must then be inserted
        ;into the bitmap for each point on the x axis.
        ;---------------------------------------------------------------

    @@:
        mov eax,clientht    ;total height available
        shr eax,1           ;->center line
        mov centerline,eax
        mov oldline,eax     ;initialize to centerline for the 1st pixel
        mov xaxis,0         ;initialize to the first point of the x axis
        sub clientht,3      ;to insure at least one space between
                            ;sine curve and top/bottom borders

sinloop:

        ;------------------------------------------------------------------
        ;compute the value for the current location on the x axis in degrees
        ;------------------------------------------------------------------
        
        mov eax,360         ;degrees in each cycle
        mul frequencyx      ;gives total number of degrees along the x axis
                            ;frequencyx is already in the MI format and the
                            ;result of multiplying by an integer will also
                            ;yield a number in the MI format which cannot
                            ;exceed the MI limits with current parameters
        mul xaxis           ;current location along the x axis
                            ;(note that although this could exceed the limits
                            ;of the MI format, the product will be divided
                            ;immediately by a larger number)
        div clientwd        ;convert into equivalent degrees

        ;--------------------------------------------------------
        ;compute the scan line on which the pixel will be located
        ;--------------------------------------------------------
        
        invoke MixSin,eax   ;get the sine of that angle
        imul clientht       ;-> an equivalent portion of the available height
        imul amplitude      ;the amplitude value in this example is realy
                            ;a percentage of the available height
                            ;imul is also used because the amplitude can be
                            ;positive or negative
        mov ecx,100
        idiv ecx            ;convert from percentage to physical value
        sar eax,1           ;only half the height is available on each side
                            ;of the center line ('sar' keeps the sign bit)

                    ;NOTE that the Mixed Integer returned by the MixSin
                    ;function was multiplied and divided by plain integers
                    ;without loosing the MI format in the results.
                            
        invoke MixToInt,eax ;gives the properly rounded integer number
        
        neg eax             ;positive values will have to be subtracted
                            ;from the center line and vice versa
        add eax,centerline  ;->scan line
        mov newline,eax

        ;--------------------------------------------------------------
        ;compute the location of the byte in the bitmap array where the 
        ;pixel is located and which bit of the byte must be changed
        ;--------------------------------------------------------------
        
        mul scanline        ;->start of scan line in array
        add eax,lpBMP       ;->its address in the bitmap
        mov edi,eax
        mov eax,xaxis       ;current position on the x axis
        mov cl,8
        div cl              ;->which equivalent byte
        mov cl,ah           ;keep remainder as a counter
        movzx eax,al        ;keep only the value in AL
        add edi,eax         ;->proper byte address
        mov al,80h          ;bit of the 1st pixel of a byte
        or  cl,cl
        jz  @F
        shr al,cl           ;shift the bit to its proper location
      @@:
        not al              ;switches that bit to a 0 and others to 1's
        and [edi],al        ;insures that bit will be a 0 to display a black
                            ;pixel without changing other pixels
        push edi
        push eax

        ;---------------------------------------------------------------
        ;If necessary, points must be added on the vertical axis between
        ;the current one and the previous one
        ;---------------------------------------------------------------

        mov updown,0        ;initialize flag for down direction
        mov ecx,newline
        sub ecx,oldline     ;get difference between last 2 pixels
        jnc @F              ;would be down direction if positive
        neg ecx             ;make difference positive
        inc updown          ;change flag to up direction
     @@:
        .if ecx > 1
            dec ecx
            shr ecx,1       ;half the distance will be filled with pixels
                            ;in line with the current pixel while the
                            ;remainder will be in line with the previous
                            ;pixel
            jnc @F
            test updown,1
            jnz @F
            inc ecx         ;for odd number of fill pixels
         @@:
            push ecx
            .if ecx != 0
                mov edx,scanline
                test updown,1
                jnz @F
                neg edx
             @@:
                add edi,edx
                and [edi],al    ;insures that bit will be a 0 to display a black
                                ;pixel without changing other pixels
                dec ecx
                jnz @B
            .endif
            mov ecx,newline
            sub ecx,oldline
            jnc @F
            neg ecx
         @@:
            dec ecx
            pop eax
            sub ecx,eax         ;gets remainder of necessary fill pixels
                                ;to line up with previous pixel
            .if ecx != 0
                mov edi,oldedi  ;retrieve location of previous pixel
                mov eax,oldal   ;retrieve the previous mask
                mov edx,scanline
                test updown,1
                jz  @F
                neg edx
             @@:
                add edi,edx
                and [edi],al    ;insures that bit will be a 0 to display a black
                                ;pixel without changing other pixels
                dec ecx
                jnz @B
            .endif
            
        .endif
        pop oldal               ;save current mask
        pop oldedi              ;save location of current pixel

        ;---------------------------------------------------
        ;Go to the next point on the x axis until terminated
        ;---------------------------------------------------

        mov eax,xaxis
        inc eax             ;next point
        cmp eax,clientwd    ;reached the end yet?
        jz  @F              ;exit loop
        mov xaxis,eax       ;update counter
        m2m oldline,newline ;current line becomes previous line
        jmp sinloop         ;continue
      @@:

        ;-----------------------------------------------------------------
        ;Create a proper bitmap from the data, select that bitmap into the
        ;compatible DC created previously, and copy the required block
        ;to the appropriate location of the window.
        ;------------------------------------------------------------------

        add clientht,3      ;back to original dimension for display
        call newscreen      ;update the client area
        pop edi
        ret

;------------------------------------------------------------------------
;This sub-routine is also used to repaint the client area when part of it
;is moved off the main screen and back on again. This is done when a 
;WM_PAINT message is received. 
;------------------------------------------------------------------------

newscreen:
        invoke CreateBitmap,clientwd,clientht,1,1,lpBMP
        mov hBMP,eax
        invoke SelectObject,hmDC,hBMP
        push eax
        invoke BitBlt,hDC,0,clienttop,clientwd,clientht,hmDC,0,0,SRCCOPY
        pop eax
        invoke SelectObject,hmDC,eax
        invoke DeleteObject,hBMP
        ret

;------------------------------------------------------------------------
;This subroutine gets the dimensions of the available client area after
;correcting for the space used by the tool bar. It also establishes the
;upper coordinate of the window where the sine graphic will be displayed.
;------------------------------------------------------------------------

clientXY:
        invoke GetClientRect,hToolBar,ADDR MainRec
        mov eax,MainRecB
        add eax,2
        mov clienttop,eax
        invoke GetClientRect,hWnd,ADDR MainRec
        m2m clientwd,MainRecR
        mov eax,MainRecB
        sub eax,clienttop
        mov clientht,eax
        ret
        
; ########################################################################

end start

