; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

    .code

; #########################################################################

MixAdd proc public MInt1:DWORD, MInt2:DWORD
        
    ; -------------------------------------------------------------
    ; Adds a Mixed Integer DWORD with another Mixed Integer DWORD
    ;
    ; eax used
    ;
    ; MInt1:    A Mixed Integer DWORD.
    ;
    ; MInt2:    A Mixed Integer DWORD.
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixAdd, var1, var2 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; resulting absolute value of the integer portion exceeds 7FFFh, the
    ; returned value would then be 80000000h (i.e. -0).
    ;
    ; If any of the two parameters is 80000000h, the returned value
    ; would also be 80000000h.
    ;
    ; N.B.  The two parameters must be in the Mixed Integer format.
    ; If the addition of a Mixed Integer format with a regular integer
    ; is required, the regular integer must first be converted to a Mixed
    ; Integer format.
    ;
    ; -------------------------------------------------------------

    mov eax,MInt2
    cmp eax,ERROR
    jnz @F
    ret
  @@:
    mov eax,MInt1
    cmp eax,ERROR
    jnz @F
    ret
  @@:
    xor eax,MInt2
    jns @F              ;jump if both variables have the same sign

;different sign parameters cannot cause an overflow by addition

    mov eax,MInt1
    add eax,MInt2
    ret

;**** The two parameters have the same sign ****

  @@:
    mov eax,MInt1
    or  eax,MInt2
    js  @F              ;jump if both variables are negative

;both variables positive
    
    mov eax,MInt1
    add eax,MInt2
    js  adderror
    ret

;both variables negative
    
  @@:
    mov eax,MInt1
    add eax,MInt2
    jns  adderror
    ret

adderror:
    mov eax,ERROR
    ret
    
MixAdd endp

; #########################################################################

end
