; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

      includelib \masm32\lib\Mix.lib

    .code

; #########################################################################

MixArctanh proc public MInt:DWORD
        
    ; -------------------------------------------------------------
    ; Computes the hyperbolic arctangent of a number per the following
    ; relationship:
    ;
    ;           arctanh(x) = 1/2 * ln[(1+x)/(1-x)]
    ;
    ; which is computed as:
    ;
    ;           arctanh(x) = 1/2 * [ln(1+x) - ln(1-x)]
    ;
    ; eax, ecx, edx used
    ;
    ; MInt:    The number in a Mixed Integer format
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixArctanh, var1 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; parameter is the error code 80000000h, it is returned unchanged.
    ;
    ; The error code would also be returned if the absolute value of the
    ; parameter is higher than or equal to 1.
    ;
    ; The MInt parameter can be positive or negative.
    ;
    ; -------------------------------------------------------------

    mov eax,MInt
    .if eax == ERROR
        ret
    .endif

    or  eax,eax
    jns @F
    neg eax

@@:
    .if eax > 0FFFFh
        mov eax,ERROR
        ret
    .endif

@@:
    mov eax,1 shl 16
    sub eax,MInt
    invoke MixLNx,eax
    push eax
    mov eax,1 shl 16
    add eax,MInt
    invoke MixLNx,eax
    pop ecx
    sub eax,ecx
    sar eax,1

    ret

MixArctanh endp

; #########################################################################

end

