; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

    .code

; #########################################################################

MixDiv proc public numerator:DWORD, denominator:DWORD
        
    ; -------------------------------------------------------------
    ; Divides a Mixed Integer DWORD by another Mixed Integer DWORD or a
    ; regular signed DWORD integer by another regular signed DWORD integer.
    ;
    ; eax, ecx, edx used
    ;
    ; numerator:    A DWORD Mixed Integer or a signed DWORD integer.
    ;
    ; denominator:  A DWORD Mixed Integer or a signed DWORD integer.
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixDiv, numerator, denominator 
    ;
    ; The result is returned in eax as a Mixed Integer format. If the
    ; absolute value of the integer portion of the quotient would be
    ; > 7FFFh the returned result would be 80000000h (i.e. -0).
    ;
    ; If any of the two parameters is 80000000h, the returned value
    ; would also be 80000000h.
    ;
    ; N.B.  The numerator and denominator must be of the same type.
    ; If the division of a Mixed Integer format by a regular integer
    ; (or vice versa) is required, the regular integer must first be
    ; converted to a Mixed Integer format to use this procedure.
    ;
    ; -------------------------------------------------------------

    mov eax,numerator
    cmp eax,ERROR
    jnz @F
    ret

  @@:
    mov ecx,denominator
    cmp ecx,ERROR
    jz  diverror

    or  ecx,ecx       ;trying to divide by 0?
    jnz @F

  diverror:
    mov eax,ERROR
    ret

  @@:
    jns @F
    neg ecx
    
  @@:
    mov edx,eax
    sar edx,16
    push edx
    or  edx,edx
    jns @F
    neg edx

  @@:
    shl edx,1
    cmp edx,ecx
    pop edx
    jae diverror            ;ERROR (would result in a division overflow)

    shl eax,16
    idiv denominator
    ret
    
MixDiv endp

; #########################################################################

end
