; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

      includelib \masm32\lib\Mix.lib

    .code

; #########################################################################

MixEx proc public MInt:DWORD
        
    ; -------------------------------------------------------------
    ; Computes the value of e raised to the power MInt per the following
    ; Taylor series after reducing the number to its fractional part:
    ;
    ;       e^x =  1 + x + x^2/2! + x^3/3! + ...
    ;
    ; eax, ecx, edx used
    ;
    ; MInt:    A positive or negative number in the Mixed Integer format
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixEx, var1 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; parameter is the error code 80000000h, it is returned unchanged.
    ; The error code will also be returned if the integer part of the
    ; result would exceed 7FFFh.
    ;
    ; -------------------------------------------------------------

    LOCAL   sums        :DWORD
    LOCAL   integer     :DWORD

    mov eax,MInt
    
    .if eax == ERROR
        ret
    .elseif eax == 0
        mov eax,10000h
        ret
    .endif

    mov ecx,eax
    or  eax,eax
    jns @F
    neg eax

  @@:
    .if eax > 0A7000h && ecx < 80000000h
        mov eax,ERROR
        ret
    .endif

    push ebx
    push esi
    push edi

    shl eax,16
    shr eax,16              ;keeps fractional part omly
    mov ebx,10000
    mul ebx
    mov sums,eax
    mov ecx,eax
    shl ebx,16
    add sums,ebx
    
    mov edi,1           ;edi holds the current factorial
    mov esi,1           ;esi holds the current number
    
  .while eax != 0       ;this loop is terminated if nothing is being added
                        ;to the accumulator

    push esi
    mov eax,ecx
  @@:
    mul ecx
    div ebx
    dec esi
    jnz @B
    pop esi

    push eax
    inc esi             ;advance to the next number
    mov eax,edi         ;copy current factorial to eax
    mul esi             ;get next factorial
    mov edi,eax         ;copy it back as current factorial
    pop eax
    xor edx,edx
    div edi
    add sums,eax        ;add this result to the accumulator

  .endw

    mov eax,sums
    mov ecx,MInt
    or  ecx,ecx
    jns @F
    neg ecx
  @@:
    shr ecx,16
        .if ecx == 0
            xor edx,edx
            shr ebx,16
            div ebx
        .elseif ecx == 1
            mov edi,MixE
            mul edi
            div ebx
        .elseif ecx == 2
            mov edi,MixE2
            mul edi
            div ebx
        .elseif ecx == 3
            mov edi,MixE3
            mul edi
            div ebx
        .elseif ecx == 4
            mov edi,MixE4
            mul edi
            div ebx
        .elseif ecx == 5
            mov edi,MixE5
            mul edi
            div ebx
        .elseif ecx == 6
            mov edi,MixE6
            mul edi
            div ebx
        .elseif ecx == 7
            mov edi,MixE7
            mul edi
            div ebx
        .elseif ecx == 8
            mov edi,MixE8
            mul edi
            div ebx
        .elseif ecx == 9
            mov edi,MixE9
            mul edi
            div ebx
        .elseif ecx == 10
            mov edi,MixE10
            mul edi
            div ebx
            .if eax >= ERROR
                mov esi,MInt
                or  esi,esi
                js @F
                mov eax,ERROR
                jmp eXit
              @@:
                xor eax,eax
                jmp eXit
            .endif
        .else 
            xor eax,eax
            jmp eXit
        .endif

    mov ecx,MInt
    or  ecx,ecx
    jns eXit

    mov ecx,eax
    mov eax,10000h
    mul eax
    div ecx
        
eXit:    
    pop edi
    pop esi
    pop ebx    
    ret
    
MixEx endp

; #########################################################################

end

