; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

      includelib \masm32\lib\Mix.lib

    .code

; #########################################################################

MixLNx proc public MInt:DWORD
        
    ; -------------------------------------------------------------
    ; Computes the natural log of a positive number per the following
    ; Taylor series after reducing the number to between 1 and e:
    ;
    ;       ln[(1+x)/(1-x)] = 2*(x + x^3/3 + x^5/5 + x^7/7 ...)
    ;
    ; eax, ecx, edx used
    ;
    ; MInt:    A positive number in the Mixed Integer format
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixLNx, var1 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; parameter is the error code 80000000h, it is returned unchanged.
    ; The error code will also be returned if the parameter is negative
    ; or zero.
    ;
    ; The result will be positive if the parameter MInt is greater
    ; than 1, negative if less than 1 and 0 if equal to 1.
    ;
    ; -------------------------------------------------------------

    LOCAL   sums        :DWORD
    LOCAL   integer     :DWORD

    mov eax,MInt
    
    .if eax >= ERROR
        mov eax,ERROR
        ret
    .elseif eax == 0
        mov eax,ERROR
        ret
    .elseif eax == 10000h
        xor eax,eax
        ret
    .endif

    push ebx
    push esi
    push edi

    mov edi,MixE
    xor esi,esi
    mov ebx,10000h

    xor edx,edx
    .if eax > ebx
        .if eax >= edi
            .if eax < MixE2
                mov esi,1
                mul ebx
                div edi
            .elseif eax < MixE3
                mov esi,2
                mul ebx
                mov ecx,MixE2
                div ecx
            .elseif eax < MixE4
                mov esi,3
                mul ebx
                mov ecx,MixE3
                div ecx
            .elseif eax < MixE5
                mov esi,4
                mul ebx
                mov ecx,MixE4
                div ecx
            .elseif eax < MixE6
                mov esi,5
                mul ebx
                mov ecx,MixE5
                div ecx
            .elseif eax < MixE7
                mov esi,6
                mul ebx
                mov ecx,MixE6
                div ecx
            .elseif eax < MixE8
                mov esi,7
                mul ebx
                mov ecx,MixE7
                div ecx
            .elseif eax < MixE9
                mov esi,8
                mul ebx
                mov ecx,MixE8
                div ecx
            .elseif eax < MixE10
                mov esi,9
                mul ebx
                mov ecx,MixE9
                div ecx
            .else
                mov esi,10
                mul ebx
                mov ecx,MixE10
                div ecx
            .endif
        .endif
        shl esi,16
                        
    .else
        .while  eax < ebx
            mul edi
            div ebx
            inc esi
        .endw
        shl esi,16
        neg esi
    .endif

    mov integer,esi
    shl ebx,4
    shl eax,4
    mov ecx,eax
    add ecx,ebx
    sub eax,ebx
    mul ebx
    div ecx
    mov sums,eax
    
    mov esi,eax
    mov edi,1

    .while eax != 0
        inc edi
        inc edi
        mov ecx,edi
        dec ecx
        mov eax,esi
      @@:
        mul esi
        div ebx
        dec ecx
        jnz @B
        xor edx,edx
        div edi
        add sums,eax
    .endw

    mov eax,sums
    shr eax,3
    jnc @F
    inc eax
  @@:
    add eax,integer

    pop edi
    pop esi
    pop ebx    
    ret
    
MixLNx endp

; #########################################################################

end

e^x = 1 + x + x^2/2! + x^3/3! + ...