; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

      includelib \masm32\lib\Mix.lib

    .code

; #########################################################################

MixSinh proc public MInt:DWORD
        
    ; -------------------------------------------------------------
    ; Computes the hyperbolic sine of a number per the following
    ; relationship:
    ;                     e^x - e^(-x)
    ;           sinh(x) = ------------
    ;                          2
    ;
    ; eax, ecx, edx used
    ;
    ; MInt:    The number in a Mixed Integer format
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixSinh, var1 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; parameter is the error code 80000000h, it is returned unchanged.
    ;
    ; The error code will also be returned if the absolute value of
    ; the parameter is higher than 11.09033 because the allowed limit
    ; of Mixed Integers would then be exceeded.
    ;
    ; The parameter can be positive or negative.
    ;
    ; -------------------------------------------------------------

    mov eax,MInt
    .if eax == ERROR
        ret
    .endif

    or  eax,eax
    jns @F
    neg eax
@@:
    .if eax > 0B1720h       ;higher than 11.09033?
        mov eax,ERROR
        ret
    .endif

    mov eax,MInt
    neg eax
    sub eax,MixLN2          ;by subtracting ln(2), the division by 2 is
                            ;performed immediately and allows results up to
                            ;the +/-32767 limit instead of +/-16383

    invoke MixEx,eax        ;-> e^(-x)
    push eax

    mov eax,MInt
    sub eax,MixLN2
    invoke MixEx,eax        ;-> e^x
    pop ecx
    sub eax,ecx
    
    ret

MixSinh endp

; #########################################################################

end

