; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

      include \masm32\include\Mix.inc

      includelib \masm32\lib\Mix.lib

    .code

; #########################################################################

MixTan proc public MInt:DWORD
        
    ; -------------------------------------------------------------
    ; Computes the sine of an angle per the following relationship:
    ;
    ;           tan(x) = sin(x)/cos(x)
    ;
    ; eax, ecx, edx used
    ;
    ; MInt:    The angle in degrees in a Mixed Integer format
    ;
    ; EXAMPLE:
    ;
    ;       invoke MixTan, var1 
    ;
    ; The result is returned in eax in a Mixed Integer format. If the
    ; parameter is the error code 80000000h, it is returned unchanged.
    ; If the result exceeds the allowed limits of Mixed Integers, the
    ; value returned in eax would be the maximum allowed.
    ;
    ; The angle provided by the parameter can be positive or negative.
    ; Angles having absolute values above 360 degrees are also allowed.
    ;
    ; N.B.  The parameter must be in the Mixed Integer format.
    ;
    ; -------------------------------------------------------------

    mov eax,MInt
    .if eax == ERROR
        ret
    .endif

    invoke MixSin,eax
    push eax

    invoke MixCos,MInt        ;cos->EAX
    pop ecx                   ;ECX=sin

    push eax
    push ecx

    invoke MixDiv,ecx,eax     ;sin(x)/cos(x)

    pop ecx                   ;ECX=sin
    pop edx                   ;EDX=cos

    .if eax == ERROR    ;due to overflow
        dec eax         ;->7fffffffh maximum positive number
        xor ecx,edx     ;to combine the sign bits of the sin and cos results
        jns @F          ;tangent is positive if both sign bits were the same
        neg eax         ;negative otherwise
    .endif

  @@:
    ret

MixTan endp

; #########################################################################

end

