; #########################################################################

  ; --------------------------------------------------
  ; This procedure was written by Raymond Filiatreault
  ; --------------------------------------------------

    .386
    .model flat, stdcall  ; 32 bit memory model
    option casemap :none  ; case sensitive

    .code

; #########################################################################

sqrt64 proc public USES ebx esi edi uIntHigh:DWORD, uIntLow:DWORD
        
    ; -------------------------------------------------------------
    ; Returns the rounded square root of a 64-bit integer. It can also
    ; return the rounded square root of a 32-bit integer or its square
    ; root with a 16-bit fraction.
    ;
    ; NOTE: If the rounded square root of a 32-bit integer is acceptable,
    ; use instead the sqrt32 function which is about 9 times faster.
    ;
    ; eax, ebx, edx used.
    ;
    ; uIntHigh: Most significant 32 bits of the integer. When multiplying
    ;           (eax) by a 32 bit integer, it would be the result in edx.
    ;
    ; uIntLow:  Least significant 32 bits of the integer. When multiplying
    ;           (eax) by a 32 bit integer, it would be the result in eax.
    ;
    ; EXAMPLE after a mul instruction by a 32-bit operand:
    ;
    ;       invoke sqrt64, edx, eax
    ;
    ; The result is in eax.
    ;
    ;
    ; EXAMPLE for the square root of a 32-bit integer when the entire
    ; fractional part must be retained:
    ;
    ;       invoke sqrt64, 32_bit_value, 0
    ;
    ; The integer portion of the square root is in the upper 16 bits
    ; of eax while the fractional part is in the lower 16 bits.
    ;
    ; 
    ; EXAMPLE for the rounded square root of a 32-bit integer:
    ;
    ;       invoke sqrt64, 0, 32_bit_value
    ;
    ; The rounded square root is returned in eax (lower 16 bits).
    ;
    ;
    ; -------------------------------------------------------------

    mov eax,uIntLow
    mov edx,uIntHigh
    or  eax,eax
    jnz @F
    or  edx,edx
    jnz @F
    
    ret                     ;if the 64-bit number=0, the root=0

  @@:
    xor edi,edi
    xor esi,esi
    mov ebx,40000000h
sqr10:
    add esi,ebx
    cmp esi,edx
    ja  sqr40
sqr20:
    sub edx,esi
    add esi,ebx
    jmp sqr50
sqr40:
    sub esi,ebx
sqr50:
    shr esi,1
    shr ebx,2
    jnz sqr10

    mov ebx,eax
    or  ebx,edx
    jnz @F
    mov eax,esi
    shl eax,16
    ret

  @@:
    mov ebx,40000000h
sqr101:
    add edi,ebx
    cmp esi,edx
    ja  sqr401
    jz  sqr301
sqr201:
    sub eax,edi
    sbb edx,esi
    add edi,ebx
    adc esi,0
    jmp sqr501
sqr301:
    cmp edi,eax
    jbe sqr201
sqr401:
    sub edi,ebx
sqr501:
    shr esi,1
    rcr edi,1
    shr ebx,2
    jnz sqr101

    or  edx,edx
    jnz sqr502
    cmp edi,eax
    jae  @F
sqr502:
    inc edi
    jnz @F
    dec edi
  @@:
    mov eax,edi
    
    ret
    
sqrt64 endp

; #########################################################################

end
